@@ -113,6 +113,12 @@ vmwareUseHWCursor(ScreenPtr pScreen, CursorPtr pCurs)
 {
     ScrnInfoPtr pScrn = infoFromScreen(pScreen);
     VMWAREPtr pVMWARE = VMWAREPTR(pScrn);
+    VmwareLog(("UseHWCursor new cursor %p refcnt %i old cursor %p refcnt %i\n",
+              pCurs, pCurs->refcnt, pVMWARE->oldCurs, pVMWARE->oldCurs ? pVMWARE->oldCurs->refcnt : 0));
+    pCurs->refcnt++;
+    if (pVMWARE->oldCurs)
+       FreeCursor(pVMWARE->oldCurs, None);
+    pVMWARE->oldCurs = pCurs;
 
     pVMWARE->hwcur.hotX = pCurs->bits->xhot;
     pVMWARE->hwcur.hotY = pCurs->bits->yhot;
@@ -141,6 +147,13 @@ static Bool
 vmwareUseHWCursorARGB(ScreenPtr pScreen, CursorPtr pCurs)
 {
     ScrnInfoPtr pScrn = infoFromScreen(pScreen);
+    VMWAREPtr pVMWARE = VMWAREPTR(pScrn);
+    VmwareLog(("UseHWCursorARGB new cursor %p refcnt %i old cursor %p refcnt %i\n",
+              pCurs, pCurs->refcnt, pVMWARE->oldCurs, pVMWARE->oldCurs ? pVMWARE->oldCurs->refcnt : 0));
+    pCurs->refcnt++;
+    if (pVMWARE->oldCurs)
+       FreeCursor(pVMWARE->oldCurs, None);
+    pVMWARE->oldCurs = pCurs;
 
     return pCurs->bits->height <= MAX_CURS &&
            pCurs->bits->width <= MAX_CURS &&
@@ -286,6 +299,7 @@ vmwareCursorInit(ScreenPtr pScreen)
         return FALSE;
 
     pVMWARE->CursorInfoRec = infoPtr;
+    pVMWARE->oldCurs = NULL;
 
     infoPtr->MaxWidth = MAX_CURS;
     infoPtr->MaxHeight = MAX_CURS;
@@ -332,6 +346,9 @@ vmwareCursorCloseScreen(ScreenPtr pScreen)
 #endif /* RENDER */
 
     vmwareHideCursor(pScrn);
+    if (pVMWARE->oldCurs)
+       FreeCursor(pVMWARE->oldCurs, None);
+    pVMWARE->oldCurs = NULL;
     xf86DestroyCursorInfoRec(pVMWARE->CursorInfoRec);
 }
 
